from typing import List

from blue_options.terminal import show_usage, xtra


def help_upload(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "".join(
        [
            "filename=<filename>",
            xtra(",mlflow,no_mlflow,~open,", mono=mono),
            "solid",
            xtra(",~warn_if_exists", mono=mono),
        ]
    )

    return show_usage(
        [
            "@upload",
            f"[{options}]",
            "[.|<object-name>]",
        ],
        "upload <object-name>.",
        mono=mono,
    )



from typing import List

from blue_options.terminal import show_usage, xtra


def help_clear(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "~dryrun"

    return show_usage(
        [
            "@storage",
            "clear",
            f"[{options}]",
        ],
        "clear storage.",
        mono=mono,
    )


def help_download_file(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@storage",
            "download_file",
            "<object-name>",
            "[filename]",
        ],
        "download filename -> <object-name>.",
        mono=mono,
    )


def help_exists(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@storage",
            "exists",
            "<object-name>",
        ],
        "True/False.",
        mono=mono,
    )


def help_list(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@storage",
            "list",
            "<prefix>",
            "[<args>]",
        ],
        "list prefix in storage.",
        mono=mono,
    )


def help_rm(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "~dryrun"

    return show_usage(
        [
            "@storage",
            "rm",
            f"[{options}]",
            "<object-name>",
        ],
        "rm <object-name>.",
        mono=mono,
    )


def help_status(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "count=<10>,depth=<2>"

    return show_usage(
        [
            "@storage",
            "status",
            f"[{options}]",
        ],
        "show storage status.",
        mono=mono,
    )


help_functions = {
    "clear": help_clear,
    "download_file": help_download_file,
    "exists": help_exists,
    "list": help_list,
    "rm": help_rm,
    "status": help_status,
}



from typing import List

from blue_options.terminal import show_usage, xtra


def help_pytest(
    tokens: List[str],
    mono: bool,
    plugin_name: str = "abcli",
) -> str:
    options = "".join(
        [
            "list",
            xtra(
                ",dryrun,~log,show_warning,~verbose",
                mono=mono,
            ),
        ]
    )

    callable = f"{plugin_name} pytest"

    if plugin_name == "abcli":
        options = f"{options},plugin=<plugin-name>"
        callable = "@pytest"

    return show_usage(
        callable.split(" ")
        + [
            f"[{options}]",
            "[filename.py|filename.py::test]",
        ],
        f"pytest {plugin_name}.",
        mono=mono,
    )



from typing import List, Dict, Callable, Union

from abcli.help.build_README import help_build_README
from abcli.help.pypi import help_functions as help_pypi
from abcli.help.pylint import help_pylint
from abcli.help.pytest import help_pytest
from abcli.help.test import help_functions as help_test


def help_functions(
    plugin_name: str = "abcli",
) -> Union[Callable, Dict[str, Union[Callable, Dict]]]:
    return {
        "build_README": lambda tokens, mono: help_build_README(
            tokens,
            mono=mono,
            plugin_name=plugin_name,
        ),
        "pypi": help_pypi(plugin_name=plugin_name),
        "pylint": lambda tokens, mono: help_pylint(
            tokens,
            mono=mono,
            plugin_name=plugin_name,
        ),
        "pytest": lambda tokens, mono: help_pytest(
            tokens,
            mono=mono,
            plugin_name=plugin_name,
        ),
        "test": help_test(plugin_name=plugin_name),
    }



from typing import List

from blue_options.terminal import show_usage, xtra


def help_blueness(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "blueness",
            "upgrade",
        ],
        "upgrade blueness 🌀.",
        mono=mono,
    )



from typing import List

from blue_options.terminal import show_usage, xtra


def help_start(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("~pull", mono=mono)

    return show_usage(
        [
            "@session",
            "start",
            f"[{options}]",
        ],
        "start a session.",
        mono=mono,
    )


help_functions = {
    "start": help_start,
}



from typing import List

from blue_options.terminal import show_usage, xtra


def help_open(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "".join(
        [
            xtra("extension=<extension>,filename=<filename>,", mono=mono),
            "QGIS",
        ]
    )

    return show_usage(
        [
            "@open",
            f"[{options}]",
            "[.|<object-name>]",
        ],
        "open <object-name>.",
        mono=mono,
    )



from typing import List

from blue_options.terminal import show_usage, xtra


def help_build(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@notebooks",
            "build",
            "[<notebook-name>]",
        ],
        "build <notebook-name>.",
        mono=mono,
    )


def help_code(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@notebooks",
            "code",
            "[<notebook-name>]",
        ],
        "code <notebook-name>.",
        mono=mono,
    )


def help_connect(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "".join(
        [
            "ip=<1-2-3-4>",
            xtra(",setup", mono=mono),
        ]
    )

    return show_usage(
        [
            "@notebooks",
            "connect",
            f"[{options}]",
        ],
        "connect to jupyter notebook on ec2:<1-2-3-4>.",
        mono=mono,
    )


def help_create(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@notebooks",
            "create | touch",
            "<notebook-name> | <path>/<notebook-name> | notebook",
        ],
        "create <notebook-name>.",
        mono=mono,
    )


def help_host(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("setup", mono=mono)

    return show_usage(
        [
            "@notebooks",
            "host",
            f"[{options}]",
        ],
        "host jupyter notebook on ec2.",
        mono=mono,
    )


def help_open(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@notebooks",
            "open",
            "[<notebook-name> | notebook]",
            "[<args>]",
        ],
        "open <notebook-name>.",
        mono=mono,
    )


help_functions = {
    "build": help_build,
    "code": help_code,
    "connect": help_connect,
    "create": help_create,
    "host": help_host,
    "open": help_open,
}



import argparse

from blueness import module
from blueness.argparse.generic import sys_exit

from abcli import NAME
from abcli.plugins.functions import (
    get_module_name,
    get_plugin_name,
    list_of_external,
    list_of_installed,
)
from abcli.logger import logger

NAME = module.name(__file__, NAME)

parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    default="",
    help="get_module_name|get_plugin_name|list_of_external|list_of_installed",
)
parser.add_argument(
    "--delim",
    type=str,
    default=", ",
)
parser.add_argument(
    "--log",
    default=1,
    type=int,
    help="0|1",
)
parser.add_argument(
    "--repo_names",
    default=0,
    type=int,
    help="0|1",
)
parser.add_argument(
    "--repo_name",
    default="",
    type=str,
)
parser.add_argument(
    "--return_path",
    default=0,
    type=int,
    help="0|1",
)
parser.add_argument(
    "--plugin_name",
    default="",
    type=str,
)
parser.add_argument(
    "--from_plugin_name",
    default=0,
    type=int,
    help="0|1",
)
args = parser.parse_args()

delim = " " if args.delim == "space" else args.delim

success = False
if args.task == "get_module_name":
    success = True
    print(get_module_name(args.repo_name))
elif args.task == "get_plugin_name":
    success = True
    print(get_plugin_name(args.repo_name))
elif args.task in ["list_of_external", "list_of_installed"]:
    output = (
        list_of_external(args.repo_names == 1)
        if args.task == "list_of_external"
        else list_of_installed(args.return_path == 1)
    )

    if args.log:
        logger.info(
            "{:,} {} {}(s): {}".format(
                len(output),
                args.task.split("list_of_")[1],
                "repo" if args.repo_names else "plugin",
                delim.join(output),
            )
        )
    else:
        print(delim.join(output))
    success = True
else:
    success = None

sys_exit(logger, NAME, args.task, success, log=args.log)



from typing import List

from blue_options.terminal import show_usage, xtra


def help_gpu_status_get(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "~from_cache"

    return show_usage(
        [
            "@gpu",
            "status",
            "get",
            f"[{options}]",
        ],
        "get gpu status.",
        mono=mono,
    )


def help_gpu_status_show(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@gpu",
            "status",
            "show",
        ],
        "show gpu status.",
        mono=mono,
    )


def help_gpu_validate(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@gpu",
            "validate",
        ],
        "validate gpu.",
        mono=mono,
    )


help_functions = {
    "status": {
        "get": help_gpu_status_get,
        "show": help_gpu_status_show,
    },
    "validate": help_gpu_validate,
}
